const rl = @import("raylib");
const std = @import("std");
const math = @import("math.zig");

const SCREENWIDTH = 1920;
const SCREENHEIGHT = 1080;
const SCALE = 50;

pub fn main() !void {
    // std.debug.print("{d}\n", .{SCREENWIDTH / 2});
    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer _ = gpa.deinit();
    const allocator = gpa.allocator();

    rl.setConfigFlags(.{ .msaa_4x_hint = true });
    rl.initWindow(SCREENWIDTH, SCREENHEIGHT, "Koordinatni sistem");
    defer rl.closeWindow();

    rl.setTargetFPS(60);
    rl.beginDrawing();

    rl.clearBackground(rl.Color.white);

    plotGraph();
    drawAxis();
    try markAxis(allocator);

    rl.endDrawing();

    while (!rl.windowShouldClose()) {
        rl.beginDrawing();
        // DEBUG
        // const pos = rl.getMousePosition();
        // std.debug.print("x: {d} y:{}\n", .{ pos.x, pos.y });
        rl.endDrawing();
    }
}

fn drawAxis() void {
    const endy_on_abscis: i32 = @divTrunc(SCREENHEIGHT, 2);
    const endx_on_ordinat: i32 = @divTrunc(SCREENWIDTH, 2);
    // x
    rl.drawLine(0, endy_on_abscis, SCREENWIDTH, endy_on_abscis, rl.Color.black);
    // y
    rl.drawLine(endx_on_ordinat, 0, endx_on_ordinat, SCREENHEIGHT, rl.Color.black);
}

fn markAxis(allocator: std.mem.Allocator) !void {
    const diff = 10;
    const font_size = 20;
    const start_x = SCREENWIDTH / 2;
    const start_y = SCREENHEIGHT / 2;
    const radius = SCALE / 15;
    var buf: [20]u8 = undefined;

    var cnt: usize = 0;
    var i: i32 = start_x;
    while (i <= SCREENWIDTH) : (i += SCALE) {
        // int to string
        const text = try std.fmt.bufPrintZ(&buf, "{}", .{cnt});
        cnt += 1;
        // 0 is due to us working with C-style strings
        var text_back = try allocator.allocSentinel(u8, text.len + 1, 0);
        defer allocator.free(text_back);
        @memcpy(text_back[0..1], "-");
        @memcpy(text_back[1..], text);

        // +x
        rl.drawCircle(@intCast(i), start_y, radius, rl.Color.black);
        rl.drawText(text, @as(i32, @intCast(i)) - diff / 2, start_y + diff, font_size, rl.Color.black);

        if (i == start_x) continue; // prevent 0 from being written/drawn twice

        // -x
        rl.drawCircle(SCREENWIDTH - i, start_y, radius, rl.Color.black);
        rl.drawText(text_back, SCREENWIDTH - i - diff - diff / 2, start_y + diff, font_size, rl.Color.black);
    }

    // start at 1 as we're not drawing 0 this time
    cnt = 1;
    i = start_y;
    while (i <= SCREENHEIGHT) : (i += SCALE) {
        // skip 0 as it is already rendered during x scale
        if (i == start_y) continue;

        const text = try std.fmt.bufPrintZ(&buf, "{}", .{cnt});
        cnt += 1;
        var text_back = try allocator.allocSentinel(u8, text.len + 1, 0);
        defer allocator.free(text_back);
        @memcpy(text_back[0..1], "-");
        @memcpy(text_back[1..], text);

        // -y
        rl.drawCircle(start_x, @intCast(i), radius, rl.Color.black);
        rl.drawText(text_back, start_x + diff, @as(i32, @intCast(i)) - diff, font_size, rl.Color.black);

        // +y
        rl.drawCircle(start_x, SCREENHEIGHT - i, radius, rl.Color.black);
        rl.drawText(text, start_x + diff, SCREENHEIGHT - i - diff, font_size, rl.Color.black);
    }
}

fn plotGraph() void {
    const diff: f32 = 0.01;
    const thickness = 2.5;
    var x_value: f32 = -SCREENWIDTH / 2;

    while (x_value <= SCREENWIDTH) : (x_value += diff) {
        const cur_pos = adjustForGridSystem(rl.Vector2{
            .x = x_value,
            .y = getY(x_value) catch {
                continue;
            },
        });

        // this is to ensure the value doesn't wrap around to zero causing the entire Y-axis to be graphed
        if (cur_pos.y > SCREENHEIGHT) {
            continue;
        }

        const next_pos = adjustForGridSystem(rl.Vector2{
            .x = x_value + diff,
            .y = getY(x_value + diff) catch {
                continue;
            },
        });

        rl.drawLineEx(cur_pos, next_pos, thickness, rl.Color.dark_blue);
    }
}

// f(x)
fn getY(x: f32) !f32 {
    return math.calculate(x, -3, .Power);
}

fn adjustForGridSystem(input_vector: rl.Vector2) rl.Vector2 {
    // std.debug.print("{d} => {d}\n", .{ input_vector.x, input_vector.x * SCALE });
    const new_vector = rl.Vector2{
        // we add screenwidth because in traditional gui libraries the grid starts in top left and not in the center like the coordinate system
        .x = input_vector.x * SCALE + SCREENWIDTH / 2,
        .y = -input_vector.y * SCALE + SCREENHEIGHT / 2, // minus is there because y works the opposite in gui libraries than in coordination system
    };

    return new_vector;
}
